import 'package:fis_common/logger/logger.dart';
import 'package:fis_measure/interfaces/date_types/rect.dart';
import 'package:fis_measure/interfaces/date_types/size.dart';

import 'point.dart';

class RectRegion {
  final double _left;
  final double _top;
  final double _right;
  final double _bottom;

  const RectRegion({
    double left = 0,
    double top = 0,
    double right = 0,
    double bottom = 0,
  })  : _left = left,
        _top = top,
        _right = right,
        _bottom = bottom;

  factory RectRegion.fill(
    double x,
    double y,
    double width,
    double height,
  ) {
    if (width < 0.0 || height < 0.0) {
      logger.e("Size_WidthAndHeightCannotBeNegative($x,$y,$width,$height)");
      width = height = 0;
    }
    final rect =
        RectRegion(left: x, top: y, right: x + width, bottom: y + height);
    return rect;
  }

  factory RectRegion.pointTLBR(
    DPoint topLeft,
    DPoint bottomRight,
  ) {
    final rect = RectRegion(
      left: topLeft.x,
      top: topLeft.y,
      right: bottomRight.x,
      bottom: bottomRight.y,
    );
    return rect;
  }

  factory RectRegion.rect(DRect rect) {
    return RectRegion(
      left: rect.left,
      top: rect.top,
      right: rect.right,
      bottom: rect.bottom,
    );
  }

  factory RectRegion.size(DSize size) {
    return RectRegion.fill(0, 0, size.width, size.height);
  }

  double get left => _left;
  double get right => _right;
  double get top => _top;
  double get bottom => _bottom;
  double get width => (_right - _left).abs();
  double get height => (_bottom - _top).abs();
  DPoint get topLeft => DPoint(left, top);
  DPoint get topRight => DPoint(right, top);
  DPoint get bottomLeft => DPoint(left, bottom);
  DPoint get bottomRight => DPoint(right, bottom);
  DPoint get center => DPoint((_left + _right) / 2, (_top + _bottom) / 2);

  /// 是否包含点
  ///
  /// [point] 指定点
  bool containsPoint(DPoint point) {
    final x = point.x, y = point.y;
    if (left > x) return false;
    if (right < x) return false;
    if (top > y) return false;
    if (bottom < y) return false;
    return true;
    // return topLeft <= point && bottomRight >= point;
  }

  @override
  bool operator ==(Object other) {
    return other is RectRegion &&
        _left == other._left &&
        _top == other._top &&
        _right == other._right &&
        _bottom == other._bottom;
  }

  @override
  int get hashCode =>
      top.hashCode ^ bottom.hashCode ^ left.hashCode ^ right.hashCode;

  @override
  String toString() {
    return 'left:$left,right:$right,top:$top,bottom:$bottom';
  }
}
