import 'package:fis_measure/configs/patient.dart';
import 'package:fis_measure/interfaces/enums/species.dart';

import 'interfaces/cardiac.dart';
import 'interfaces/general.dart';
import 'interfaces/obstetrics.dart';
import 'interfaces/urology.dart';
import 'common/cardiac.dart';
import 'common/general.dart';
import 'common/obstetrics.dart';
import 'common/urology.dart';
import 'mouse/cardiac.dart';
import 'mouse/general.dart';
import 'mouse/obstetrics.dart';
import 'mouse/urology.dart';

abstract class UnitFormulas {
  static final _CommonProxy _commonProxy = _CommonProxy();
  static final _MouseProxy _mouseProxy = _MouseProxy();
  static bool get isMouse =>
      GlobalPatientConfig.speciesType == SpeciesType.mouse;
  static _InstanceProxy get _instance {
    return isMouse ? _mouseProxy : _commonProxy;
  }

  static ICardiacFormulas get cardiac => _instance.cardiac;
  static IGeneralFormulas get general => _instance.general;
  static IObstetricsFormulas get obstetrics => _instance.obstetrics;
  static IUrologyFormulas get urology => _instance.urology;
}

abstract class _InstanceProxy {
  late final ICardiacFormulas cardiac;
  late final IGeneralFormulas general;
  late final IObstetricsFormulas obstetrics;
  late final IUrologyFormulas urology;
}

class _CommonProxy extends _InstanceProxy {
  _CommonProxy() {
    cardiac = CardiacFormulasCommon();
    general = GeneralFormulasCommon();
    obstetrics = ObstetricsFormulasCommon();
    urology = UrologyFormulasCommon();
  }
}

class _MouseProxy extends _InstanceProxy {
  _MouseProxy() {
    cardiac = CardiacFormulasMouse();
    general = GeneralFormulasMouse();
    obstetrics = ObstetricsFormulasMouse();
    urology = UrologyFormulasMouse();
  }
}
