import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_trace_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_trace_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'dart:ui' as ui;
import 'dart:convert';
import 'dart:typed_data';

import 'package:vid/us/vid_us_unit.dart';

class URMTracePerfusionCal extends URMTraceMeasureCal {
  URMTracePerfusionCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMPerfusion,
        rOIType: URMROIType.URMTrace,
        srcDPoints: param!.srcDPoints,
      );
      if (outresult == null) {
        return;
      }
      final feature = ref.feature!;

      if (ref.meta.outputs.isNotEmpty) {
        final first = ref.meta.outputs.first;
        first.unit = VidUsUnit.None;
        if (first == null) return;

        feature.updateFloatValue(first, outresult.resultData, first.unit);

        if (feature is! TracePerfusionImageFeature) return;
        if (outresult.perfusionImgBase64 != null) {
          feature.perfusionImg =
              await loadImageFromBase64(outresult.perfusionImgBase64!);
          feature.leftTopPoint = urmApplication
              .viewToLocal(outresult.perfusionScaleDRect!.topLeft!);
          feature.rightBottomPoint = urmApplication
              .viewToLocal(outresult.perfusionScaleDRect!.bottomRight!);
        }
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }

  Future<ui.Image> loadImageFromBase64(String base64String) async {
    Uint8List bytes = base64Decode(base64String);
    ui.Codec codec = await ui.instantiateImageCodec(bytes);
    ui.FrameInfo frameInfo = await codec.getNextFrame();
    return frameInfo.image;
  }
}
