import 'package:fis_measure/define.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';

class AiModifierModeChangeButton extends FStatefulWidget
    implements FInteractiveContainer {
  const AiModifierModeChangeButton(
      {super.key,
      required this.onClick,
      required this.modeName,
      required this.isSelected,
      required this.mainColor});
  final VoidCallback onClick;
  final String modeName;
  final bool isSelected;
  final Color mainColor;

  @override
  FState<AiModifierModeChangeButton> createState() => _MyScheduleItemState();

  @override
  String get pageName => "MyScheduleItem";
}

class _MyScheduleItemState extends FState<AiModifierModeChangeButton> {
  bool _isHover = false;

  @override
  FWidget build(BuildContext context) {
    return FMouseRegion(
      cursor: SystemMouseCursors.click,
      onEnter: _handleMouseEnter,
      onExit: _handleMouseExit,
      child: FGestureDetector(
        name: "myScheduleItem",
        businessParent: widget,
        onTap: () {
          widget.onClick.call();
        },
        child: FContainer(
          height: 40,
          padding: const EdgeInsets.symmetric(horizontal: 5),
          decoration: BoxDecoration(
            color: _isHover
                ? widget.mainColor.withOpacity(0.08)
                : Colors.transparent,
            borderRadius: BorderRadius.circular(5),
          ),
          child: FRow(
            children: [
              FCenter(
                child: QuickFWidget(
                  Checkbox(
                    splashRadius: 0,
                    side:
                        BorderSide(color: widget.mainColor, width: 1), // 勾勾的颜色
                    activeColor: widget.mainColor, // 勾勾的背景色
                    shape: const CircleBorder(),
                    value: widget.isSelected,
                    onChanged: (v) {
                      widget.onClick.call();
                    },
                  ),
                ),
              ),
              const FSizedBox(width: 5),
              FCenter(
                child: FText(
                  widget.modeName,
                  style: const TextStyle(color: Colors.white),
                ),
              ),
              FExpanded(
                child: FContainer(),
              ),
              const FSizedBox(width: 10),
            ],
          ),
        ),
      ),
    );
  }

  void _handleMouseEnter(PointerEvent details) {
    setState(() {
      _isHover = true;
    });
  }

  void _handleMouseExit(PointerEvent details) {
    setState(() {
      _isHover = false;
    });
  }
}
