import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const TEXT_FIELD_FILL_COLOR = Color.fromARGB(255, 251, 249, 249);
const TEXT_FIELD_BORDER_ERROR_COLOR = Color(0xFFEF5350);

class FBorderTextFormField extends StatefulWidget implements FWidget {
  const FBorderTextFormField({
    Key? key,
    this.hintText,
    this.initialValue,
    this.focusNode,
    this.controller,
    this.onChanged,
    this.maxLength,
    this.obscureText = false,
    this.inputFormatters,
    this.inError = false,
    this.isShowVisibilityIcon = false,
  }) : super(key: key);
  final String? hintText;
  final String? initialValue;
  final FocusNode? focusNode;
  final TextEditingController? controller;
  final ValueChanged<String>? onChanged;
  final int? maxLength;
  final bool obscureText;
  final List<TextInputFormatter>? inputFormatters;
  final bool inError;
  final bool isShowVisibilityIcon;

  @override
  State<StatefulWidget> createState() {
    return _FBorderTextFormFieldState();
  }

  static InputBorder createDesktopBorder(bool inError) {
    final border = OutlineInputBorder(
      borderRadius: const BorderRadius.all(Radius.circular(8)),
      borderSide: BorderSide(
        color: inError
            ? TEXT_FIELD_BORDER_ERROR_COLOR
            : FTheme.ins.data.colorScheme.line,
      ),
    );
    return border;
  }

  static EdgeInsets createDesktopContentPadding() {
    return const EdgeInsets.symmetric(vertical: 16, horizontal: 34);
  }
}

class _FBorderTextFormFieldState extends State<FBorderTextFormField> {
  bool _obscureText = false;

  @override
  void initState() {
    _obscureText = widget.obscureText;
    if (mounted) {
      setState(() {});
    }
    super.initState();
  }

  @override
  FWidget build(BuildContext context) {
    if (kIsMobile) {
      final border = OutlineInputBorder(
        borderRadius: const BorderRadius.all(Radius.circular(4)),
        borderSide: BorderSide(
          color: widget.inError
              ? TEXT_FIELD_BORDER_ERROR_COLOR
              : FTheme.ins.data.colorScheme.line,
        ),
      );
      return FTextFormField(
        initialValue: widget.initialValue,
        focusNode: widget.focusNode,
        controller: widget.controller,
        maxLength: widget.maxLength,
        obscureText: _obscureText,
        obscuringCharacter: '･',
        onChanged: widget.onChanged,
        inputFormatters: widget.inputFormatters,
        decoration: InputDecoration(
          fillColor: TEXT_FIELD_FILL_COLOR,
          filled: true,
          hintText: widget.hintText,
          hintStyle: const TextStyle(
            fontSize: 14,
            color: Colors.black54,
          ),
          alignLabelWithHint: true,
          counterText: '',
          isDense: true,
          isCollapsed: true,
          contentPadding:
              const EdgeInsets.symmetric(vertical: 8, horizontal: 26),
          enabledBorder: border,
          focusedBorder: border,
          suffixIconConstraints: BoxConstraints.tight(const Size(48, 24)),
          suffixIcon: widget.isShowVisibilityIcon
              ? GestureDetector(
                  onTap: () {
                    setState(() {
                      _obscureText = !_obscureText;
                    });
                  },
                  child: Icon(
                    _obscureText ? Icons.visibility_off : Icons.visibility,
                  ),
                )
              : null,
        ),
      );
    }

    final border = FBorderTextFormField.createDesktopBorder(widget.inError);
    return FTextFormField(
      initialValue: widget.initialValue,
      focusNode: widget.focusNode,
      controller: widget.controller,
      maxLength: widget.maxLength,
      obscureText: _obscureText,
      obscuringCharacter: '･',
      onChanged: widget.onChanged,
      inputFormatters: widget.inputFormatters,
      style: const TextStyle(fontSize: 18),
      decoration: InputDecoration(
        fillColor: TEXT_FIELD_FILL_COLOR,
        filled: true,
        hintText: widget.hintText,
        hintStyle: const TextStyle(
          fontSize: 16,
          color: Colors.black54,
        ),
        suffixIconConstraints: BoxConstraints.tight(const Size(48, 24)),
        suffixIcon: widget.isShowVisibilityIcon
            ? GestureDetector(
                onTap: () {
                  setState(() {
                    _obscureText = !_obscureText;
                  });
                },
                child: Icon(
                  _obscureText ? Icons.visibility_off : Icons.visibility,
                ),
              )
            : null,
        alignLabelWithHint: true,
        counterText: '',
        contentPadding: FBorderTextFormField.createDesktopContentPadding(),
        enabledBorder: border,
        focusedBorder: border,
      ),
    );
  }
}
