import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';

/// 注释项
///
// ignore: must_be_immutable
class MobileAnnotationSelector extends StatefulWidget implements FWidget {
  const MobileAnnotationSelector({Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() => _MobileAnnotationSelectorState();
}

class _MobileAnnotationSelectorState extends State<MobileAnnotationSelector> {
  // ignore: non_constant_identifier_names
  // static final C_SUPPORTED_TEXTS = measureData.annotationList;

  final scrollController = ScrollController();
  final application = Get.find<IApplication>();
  late final measureHandler = Get.find<IMeasureHandler>();

  /// 数据
  late final measureData = Get.find<MeasureDataController>();

  bool isArrowMode = false;

  void onChangedAnnotationType(
    Object sender,
    AnnotationType? e,
  ) {
    setState(() {
      isArrowMode =
          measureHandler.changedAnnotationType == AnnotationType.arrow;
    });
  }

  @override
  void initState() {
    measureHandler.onChangedAnnotationType.addListener(onChangedAnnotationType);

    super.initState();
  }

  @override
  void dispose() {
    measureHandler.onChangedAnnotationType
        .removeListener(onChangedAnnotationType);

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        const SizedBox(
          height: 50,
        ),
        _buildModeChangeBtn(),
        const SizedBox(
          height: 10,
        ),
        Expanded(
          child: SizedBox(
            width: 130,
            child: Scrollbar(
              controller: scrollController,
              isAlwaysShown: true,
              child: GridView.count(
                controller: scrollController,
                crossAxisCount: 1,
                childAspectRatio: 3,
                children: measureData.annotationList.map((e) {
                  return _buildAnnotationTools(e);
                }).toList(),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildAnnotationTools(String tools) {
    const style = TextStyle(color: Colors.white, fontSize: 12);
    final dragStyle = TextStyle(
        color: Colors.amber,
        fontSize: 14 * Get.find<IApplication>().displayScaleRatio);
    return Row(
      mainAxisSize: MainAxisSize.max,
      children: [
        Expanded(
          child: Draggable<String>(
            data: tools,
            affinity: Axis.horizontal,
            dragAnchorStrategy: pointerDragAnchorStrategy,
            child: OutlinedButton(
              child: Text(tools, style: style),
              onPressed: () {
                PromptBox.toast(i18nBook.measure.dragLeftAddComment.t);
              },
              style: OutlinedButton.styleFrom(
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(4),
                ),
                side: const BorderSide(
                  color: Color.fromRGBO(124, 124, 124, 1),
                ),
                fixedSize: const Size.fromHeight(40),
              ),
            ),
            feedback: Material(
              color: Colors.transparent,
              child: Text(tools, style: dragStyle),
            ),
            onDragStarted: () {
              HapticFeedback.lightImpact();
              measureHandler.changedAnnotationType = AnnotationType.label;
              application.switchAnnotation(AnnotationType.label, tools);
            },
          ),
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }

  Widget _buildModeChangeBtn() {
    return SizedBox(
      width: i18nBook.isCurrentRussian ? 160 : 150,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          InkWell(
            child: _buildModeBtn(!isArrowMode, i18nBook.measure.text.t,
                Icons.insert_comment_outlined, true),
            onTap: () {
              if (measureHandler.changedAnnotationType !=
                  AnnotationType.label) {
                HapticFeedback.lightImpact();
                measureHandler.changedAnnotationType = AnnotationType.label;
                application.switchAnnotation(AnnotationType.label);
              }
            },
          ),
          InkWell(
            child: _buildModeBtn(
              isArrowMode,
              i18nBook.measure.arrowhead.t,
              Icons.compare_arrows_rounded,
              false,
            ),
            onTap: () {
              if (measureHandler.changedAnnotationType !=
                  AnnotationType.arrow) {
                HapticFeedback.lightImpact();
                measureHandler.changedAnnotationType = AnnotationType.arrow;
                application.switchAnnotation(AnnotationType.arrow);
              }
            },
          ),
        ],
      ),
    );
  }

  Widget _buildModeBtn(
      bool ifActive, String text, IconData iconData, bool ifLeft) {
    const style = TextStyle(color: Colors.grey, fontSize: 12);
    const activeStyle = TextStyle(color: Colors.white, fontSize: 12);
    const borderColor = Colors.grey;
    const activeBorderColor = Colors.white;
    const borderSide = BorderSide(color: borderColor);
    const activeBorderSide = BorderSide(color: activeBorderColor);
    const leftBorderStyle = BorderRadius.only(
      topLeft: Radius.circular(4),
      bottomLeft: Radius.circular(4),
    );
    const rightBorderStyle = BorderRadius.only(
      topRight: Radius.circular(4),
      bottomRight: Radius.circular(4),
    );
    return Container(
      decoration: BoxDecoration(
        borderRadius: ifLeft ? leftBorderStyle : rightBorderStyle,
        border: Border.fromBorderSide(ifActive ? activeBorderSide : borderSide),
      ),
      padding: const EdgeInsets.symmetric(
        horizontal: 4,
        vertical: 4,
      ),
      child: Row(
        children: [
          Icon(
            iconData,
            color: ifActive ? Colors.white : Colors.grey,
          ),
          const FSizedBox(
            width: 6,
          ),
          Text(text, style: ifActive ? activeStyle : style),
        ],
      ),
    );
  }
}
