import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class LoadingAnimatedIconButton extends StatefulWidget {
  const LoadingAnimatedIconButton({
    Key? key,
    required this.icon,
    required this.onPressed,
    this.animationDuration = 1000,
  }) : super(key: key);
  final IconData icon;
  final double animationDuration;
  final VoidCallback onPressed;
  @override
  State<StatefulWidget> createState() => _LoadingAnimatedIconButtonState();
}

class _LoadingAnimatedIconButtonState extends State<LoadingAnimatedIconButton> {
  bool _isLoading = false;
  @override
  Widget build(BuildContext context) {
    return IconButton(
      splashRadius: 2,
      padding: EdgeInsets.zero,
      onPressed: () {
        if (!_isLoading) {
          HapticFeedback.lightImpact();
          widget.onPressed();
          setState(() {
            _isLoading = true;
          });
          Future.delayed(
              Duration(milliseconds: widget.animationDuration.toInt()), () {
            setState(() {
              _isLoading = false;
            });
          });
        }
      },
      icon: _isLoading
          ? const SizedBox(
              width: 20,
              height: 20,
              child: CircularProgressIndicator(
                strokeWidth: 2,
                color: Colors.white,
              ))
          : Icon(
              widget.icon,
              color: Colors.white,
            ),
    );
  }
}
